'''
  Copyleft ITEG IT-Engineers GmbH 2025
'''

#import argparse
import getopt
#import os
import os.path
import re
import subprocess
import sys
import xml.etree.ElementTree as ET

from clazzes.pba import dto
from clazzes.pba import pbalogging
from clazzes.pba import helpers
from dataclasses import dataclass

log = pbalogging.getLogger()

# global work and functions based on tis/tisgraph/v2/tisgraph-ts/src/artwork/icons/generate_sprites.py

ET.register_namespace("", "http://www.w3.org/2000/svg")
ET.register_namespace("xlink", "http://www.w3.org/1999/xlink")

scour_child_cli_args = [
    "scour",
    "--enable-id-stripping",
    "--enable-viewboxing",
    "--shorten-ids",
    "--enable-comment-stripping",
    "--create-groups",
    "--remove-metadata",
    "--no-line-breaks",
    "--strip-xml-space",
]

scour_sprites_cli_args = [
    "scour",
    "--enable-viewboxing",
    "--enable-comment-stripping",
    "--create-groups",
    "--remove-metadata",
    "--no-line-breaks",
    "--protect-ids-prefix", "v-",
    "--protect-ids-prefix", "s-",
]

padding = 4

known_attributes = {"version","viewBox","width","height"}

name_re1 = re.compile(r"([A-Z])")
name_re2 = re.compile(r"[-_]")
name_re3 = re.compile(r"\s+")

def transform_name(name: str) -> str:
    step1 = name_re1.sub(r" \1", name).lower()
    step2 = name_re2.sub(" ", step1).strip()
    step3 = name_re3.sub("-", step2)
    log.debug(step3)
    return step3

size_re = re.compile(r"^(.*?)(\d+)x(\d+)$")

def optimize_sprite_svg(svg_string: str, i: int) -> str:
    log.debug("For scour_one feeding svg_string into scour")
    #scour_one = subprocess.run(scour_child_cli_args + ["--shorten-ids-prefix", f"i{i}"], check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True, input=svg_string).stdout
    scourRc, scour_one, scourErrors = helpers.performPipeDebugLogged(scour_child_cli_args + ["--shorten-ids-prefix", f"i{i}"], input=svg_string)
    #log.debug("Back from feeding scour_one into scour")
    if scourRc != 0:
        log.error("Scour for svg_string to scour_one gave us bad returncode %d, aborting. scour's stderr was:\n%s" % (scourRc, scourErrors))
        sys.exit(scourRc)
    log.debug("For scour_two feeding scour_one into scour")
    #scour_two = subprocess.run(scour_child_cli_args + ["--shorten-ids-prefix", f"i{i}"], check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True, input=scour_one).stdout
    scourRc, scour_two, scourErrors = helpers.performPipeDebugLogged(scour_child_cli_args + ["--shorten-ids-prefix", f"i{i}"], input=scour_one)
    if scourRc != 0:
        log.error("Scour for scour_one to scour_two gave us bad returncode %d, aborting. scour's stderr was:\n%s" % (scourRc, scourErrors))
        sys.exit(scourRc)
    #log.debug("Back from feeding scour_two into scour")
    return scour_two

def list_input_directory(input: str) -> list[str]:
    return [
        os.path.join(input, file_name)
        for file_name in os.listdir(input)
        if file_name.endswith(".svg")
        and not "sprite" in file_name
        and not "template" in file_name
        and os.path.isfile(os.path.join(input, file_name))
    ]

def list_input_files(inputs: list[str]) -> list[str]:
    ret = []
    for input in inputs:
        if os.path.isdir(input):
            ret.extend(list_input_directory(input))
        else:
            ret.append(input)

    return ret

@dataclass
class ProcessReturn:
    width: float
    height: float

    defs: list[ET.Element]
    main_children: list[ET.Element]

def process_single_item(path: str, i: int, x: float) -> ProcessReturn:
    with open(path, "r") as fp:
        unoptimized_svg_string = fp.read()

    svg_tree = ET.fromstring(optimize_sprite_svg(unoptimized_svg_string, i))


    for key in svg_tree.keys():
        if not key in known_attributes:
            raise Exception(f"Unknown root svg attribute {key}")

    no_ext = os.path.splitext(os.path.basename(path))[0]

    sz_match = size_re.match(no_ext)

    base_name = no_ext if sz_match is None else sz_match[1]

    remaining_id = transform_name(base_name)

    [vbx,vby,vbwidth,vbheight] = map(float, svg_tree.attrib["viewBox"].split())

    sz_width = vbwidth if sz_match is None else float(sz_match[2])
    sz_height = vbheight if sz_match is None else float(sz_match[3])

    symbol_tree = ET.Element("{http://www.w3.org/2000/svg}symbol", {
        "viewBox": svg_tree.attrib["viewBox"],
        "id": "s-" + remaining_id,
        "overflow": "hidden",
    })


    symbol_tree.extend(list(svg_tree))


    view = ET.Element("{http://www.w3.org/2000/svg}view", {
        "id": "v-" + remaining_id,
        "viewBox": f"{x} 0 {sz_width} {sz_height}",
        #"width": str(sz_width),
        #"height": str(sz_height),
    })

    use = ET.Element("{http://www.w3.org/2000/svg}use", {
        "{http://www.w3.org/1999/xlink}href": "#s-" + remaining_id,
        "x": str(x),
        "y": "0",
        "width": str(sz_width),
        "height": str(sz_height),
        "overflow": "hidden",
    })

    return ProcessReturn(
        width=sz_width,
        height=sz_height,

        defs=[symbol_tree],
        main_children=[view, use],
    )



def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...] inputFile... outputFile\n" % argv0
    usage += "\n"
    usage += "Reads the SVG file(s) specified as inputFile(s) and combines them\n"
    usage += "into the SVG file specified as outputFile.\n"
    usage += pbalogging.getCommonUsageOptions()
    usage += "  Specific options are:\n"
    usage += "\n"
    return usage

    
def run_pba_generate_svg_sprites(argv0,args) -> int:
    
    try:
        opts, filePaths = getopt.getopt(args, "dqtv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    pbaConfiguration = dto.PBAConfiguration()

    if filePaths == None:
        print("No input or output file paths specified\n", file=sys.stderr)
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(1)
        
    nrFilePaths = len(filePaths)
    if nrFilePaths < 1:
        print("No input or output file paths specified\n", file=sys.stderr)
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(1)
    if nrFilePaths < 2:
        print("Only %d file path(s) specified, we need at least one inputFile and one outputFile" % nrFilePaths, file=sys.stderr)
        sys.exit(1)
        
    outputFileName = filePaths.pop()
    nrInputFiles = len(filePaths)
    log.notice("Going to process %d input files" % nrInputFiles)
    if os.path.exists(outputFileName):
        log.warning("Going to overwrite existing %s" % outputFileName)
    else:
        log.notice("Going to create %s" % outputFileName)

    # go to work, based on tis/tisgraph/v2/tisgraph-ts/src/artwork/icons/generate_sprites.py
    
    output = ET.Element("{http://www.w3.org/2000/svg}svg", {
        "version": "1.1",
        # workaround webkit and blink bug.
        "preserveAspectRatio": "none",
    })

    defs = ET.SubElement(output, "defs")
    x = 0.
    height = 0.

    nrFilesRead = 0
    for (i, path) in enumerate(list_input_files(filePaths)):
        if path == outputFileName:
            log.warning("Ignoring input file that equals output file, i.e. %s" % path)
            nrInputFiles -= 1
            continue
        if not os.path.exists(path):
            log.warning("Ignoring missing input file %s" % path)
            continue
        
        log.info("Reading %s ..." % path)
        nrFilesRead += 1

        process_ret = process_single_item(path, i, x)

        x += process_ret.width + padding
        height = max(height, process_ret.height)

        defs.extend(process_ret.defs)
        output.extend(process_ret.main_children)

    if nrFilesRead == 0:
        log.error("No input file found, see above for attempted paths")
        sys.exit(1)
        
    output.attrib["viewBox"] = f"0 0 {x} {height}"

    out_string = ET.tostring(output, encoding="utf-8", xml_declaration=True)

    log.debug("Feeding out_string into scour")
    #completed_svg_string = subprocess.run(scour_sprites_cli_args, stdout=subprocess.PIPE, text=False, input=out_string).stdout
    scourRc, completed_svg_string, scourErrors = helpers.performPipeDebugLogged(scour_sprites_cli_args, input=out_string, text=False)
    if scourRc != 0:
        log.error("Scour for out_string to completed_svg_string gave us bad returncode %d, aborting. scour's stderr was:\n%s" % (scourRc, scourErrors))
        sys.exit(scourRc)
    #log.debug("Back from feeding out_string into scour.")

    log.info("Writing %s ..." % outputFileName)
    with open(outputFileName, "wb") as fp:
        fp.write(completed_svg_string)
    
    if nrFilesRead < nrInputFiles:
        log.warning("Not all %d input files have been found & read, only %d" % (nrInputFiles, nrFilesRead))
        log.info("Done combining %d input files into %s" % (nrFilesRead, outputFileName))
    else:
        log.good("Done combining all %d input files into %s" % (nrInputFiles, outputFileName))

    sys.exit(0)

