'''
  $Id: pba_mvn_javadoc_publish.py 158 2021-02-18 20:35:39Z cl_yubikey_2018a $
  
  Copyleft ITEG IT-Engineers GmbH 2022
'''

import getopt
#import glob
#import io
import os
#import os.path
import subprocess
import sys

from clazzes.pba import dto
from clazzes.pba import pbalogging

log = pbalogging.getLogger()


def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...] [pomXml]\n" % argv0
    usage += "\n"
    usage += "Parses ./pom.xml to compute eventual rsync targets for the javadoc,\n"
    usage += "using the deployment target + javadoc + groupId + artifactId,\n"
    usage += "optionally postfixed with '_' and a branch name specified with option -b.\n"
    usage += "\n"
    usage += "Then asks wether to sync the generated javadoc(s) to the rsync target(s),\n"
    usage += "unless option -p was specified.\n"
    usage += "\n"
    usage += "Then, if '-m' was specified, calls 'mvn package javadoc:javadoc'\n"
    usage += "\n"
    usage += "Finally performs an rsync-over-ssh call to publish the local javadoc.\n"
    usage += pbalogging.getCommonUsageOptions()
    usage += "  Specific options are:\n"
    usage += "    -b bn  Append specified bn (branch name) to sync target.\n"
    usage += "           It is recommended to start that with '_' or '-' or '.'.\n"
    usage += "    -m     Call 'mvn package javadoc:javadoc' before syncing.\n"
    usage += "    -p     Publish the created javadoc without checking back.\n"
    usage += "\n"
    usage += "If no pom.xml is specified it is expected in the current directory.\n"
    usage += "\n"
    return usage


def run_pba_mvn_javadoc_publish(argv0,args):
    
    try:
        opts, pomXmlPaths = getopt.getopt(args, "b:dhmpqtv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    pbaConfiguration = dto.PBAConfiguration()

    performMavenJavadoc = False
    publishWithoutBackcheck = False
    branchNamePostfix = ""

    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        elif o in ["-b"]:
            branchNamePostfix = a
        elif o in ["-m"]:
            performMavenJavadoc = True
        elif o in ["-p"]:
            publishWithoutBackcheck = True
        else:
            log.warning("Unhandled option: %s %s" %
                  (o, a))
            publishWithoutBackcheck = False

    if len(pomXmlPaths) > 1:
        log.error("Multiple pom.xml paths specified, only one allowed currently (may change in the future).")
        sys.exit(1)

    pomXmlPath = ""
    if pomXmlPaths == None:
        pomXmlPath = "%s/%s" % (os.getcwd(), "pom.xml")
    elif len(pomXmlPaths) == 0:
        pomXmlPath = "%s/%s" % (os.getcwd(), "pom.xml")
    else:
        pomXmlPath = pomXmlPaths[0]
    
    if not os.path.exists(pomXmlPath):
        log.error("Current directory does not contain pom.xml, no %s" % pomXmlPath)
        sys.exit(1)

    log.info("Parsing %s ..." % pomXmlPath)
    pomXml = dto.PomXml(location=pomXmlPath, pbaConfiguration=pbaConfiguration)
    pomXml.parsePomXml(pomXmlPath)

    pomXmlsToProcess = []
    if pomXml.modulesPomXmls == None:
        # single module project
        #log.debug("Single module repo URLs are %s and %s" % (pomXml.releaseRepositoryUrl, pomXml.snapshotRepositoryUrl))
        if pomXml.javaDocSyncTarget == None:
            log.error("Single module project without javaDoc sync target: %s" % pomXmlPath)
            sys.exit(1)
        pomXmlsToProcess.append(pomXml)
        if len(branchNamePostfix) > 0:
            log.debug("Appending branch name postfix '%s' to sync target of %s" % (branchNamePostfix, pomXml.path))
            pomXml.javaDocSyncTarget.syncTarget += branchNamePostfix
        log.info("Single module project %s\n  to sync javaDoc\n  from %s\n  to %s" % (
            pomXmlPath, pomXml.javaDocSyncTarget.syncSource, pomXml.javaDocSyncTarget.syncTarget))
    elif len(pomXml.modulesPomXmls) > 0:
        #log.debug("Main module repo URLs are %s and %s" % (pomXml.releaseRepositoryUrl, pomXml.snapshotRepositoryUrl))
        if pomXml.javaDocSyncTarget != None:
            log.debug("Expectable non-error: Main module has no javaDoc sync target: %s" % pomXmlPath)
        else:
            log.debug("Main module project %s has javaDoc sync target URL %s resp. %s" % (
                pomXmlPath, pomXml.javaDocSyncTarget.repoUrl, pomXml.javaDocSyncTarget.describe()))
        # multi module project
        for modulePomXml in pomXml.modulesPomXmls:
            if modulePomXml.javaDocSyncTarget == None:
                log.error("Module without javaDoc sync target: %s" % modulePomXml.path)
                sys.exit(1)
            pomXmlsToProcess.append(modulePomXml)
            if len(branchNamePostfix) > 0:
                log.debug("Appending branch name postfix '%s' to sync target of %s" % (branchNamePostfix, modulePomXml.path))
                modulePomXml.javaDocSyncTarget.syncTarget.append(branchNamePostfix)
            log.info("Module %s\n  to sync javaDoc from %s\n  to %s" % (
                modulePomXml.path, modulePomXml.javaDocSyncTarget.syncSource, modulePomXml.javaDocSyncTarget.syncTarget))
    else:
        log.error("Multi module project with empty module list: %s" % pomXmlPath)
        sys.exit(1)

    moduleCount = len(pomXmlsToProcess)
        
    if publishWithoutBackcheck:
        if performMavenJavadoc:
            log.info("Skipping backcheck due to '-p', going to create and sync javadoc for %d modules ..." % moduleCount)
        else:
            log.info("Skipping backcheck due to '-p', going to sync javadoc already created for %d modules ..." % moduleCount)
    else:
        if performMavenJavadoc:
            choice = input("OK to create javaDoc and on success perform the %s syncs above?\n"
                           "If your are You sure it is ok to continue, enter 'y' or 'yes' ... " % moduleCount)
        else:
            choice = input("OK to perform the %s syncs above for javadoc already created?\n"
                           "If your are You sure it is ok to continue, enter 'y' or 'yes' ... " % moduleCount)
        if not choice in ["y", "yes"]:
            log.error("You chose to abort.")
            sys.exit(1)

    # javadoc call
    if performMavenJavadoc:
        if not "JAVA_HOME" in os.environ:
            os.environ["JAVA_HOME"] = "/usr/lib/jvm/java-11-openjdk-amd64"
            log.warning("Setting missing environment variable JAVA_HOME to %s" % os.environ["JAVA_HOME"])
        mainModuleDir = os.path.dirname(pomXml.path)
        os.chdir(mainModuleDir)
        log.info("Performing mvn package javadoc:javadoc in %s" % mainModuleDir)
        cp = subprocess.run(["/usr/bin/mvn", "package", "javadoc:javadoc"])
        if cp.returncode != 0:
            log.error("Failed to mvn package javadoc:javadoc in %s" % mainModuleDir)
            sys.exit(1) # or continue?
        log.good("Successfully performed mvn package javadoc:javadoc in %s" % mainModuleDir)
    else:
        log.notice("Not performing mvn package javadoc:javadoc, because option '-m' was not specified.")

    for pomXmlToProcess in pomXmlsToProcess:
        if not os.path.isdir(pomXmlToProcess.javaDocSyncTarget.syncSource):
            log.error("Local javadoc directory missing: %s" % pomXmlToProcess.javaDocSyncTarget.syncSource)
            sys.exit(1) # or continue?

    # mkdir
    log.notice("If the following ssh-based commands seem to fail silently you might need to accept the ssh host key.")
    for pomXmlToProcess in pomXmlsToProcess:
        log.notice("Making sure directory %s@%s exists ..." % (pomXmlToProcess.javaDocSyncTarget.repoUser, pomXmlToProcess.javaDocSyncTarget.syncTarget))
        cp = subprocess.run(["/usr/bin/ssh",
                             "%s@%s" % (pomXmlToProcess.javaDocSyncTarget.repoUser, pomXmlToProcess.javaDocSyncTarget.repoServer),
                             "/bin/mkdir",
                             "-pv",
                             pomXmlToProcess.javaDocSyncTarget.repoJavaDocDir])
        if cp.returncode != 0:
            log.error("Failed to make sure directory %s exists ..." % pomXmlToProcess.javaDocSyncTarget.syncTarget)
            sys.exit(1) # or continue?
        log.good("Successfully made sure directory %s exists ..." % pomXmlToProcess.javaDocSyncTarget.syncTarget)
    
    # xsync
    for pomXmlToProcess in pomXmlsToProcess:
        log.notice("Calling rsync %s %s@%s ..." % (
            pomXmlToProcess.javaDocSyncTarget.syncSource,
            pomXmlToProcess.javaDocSyncTarget.repoUser,
            pomXmlToProcess.javaDocSyncTarget.syncTarget))
        # rsync options are from ITEG's wrapper, which has been dropped because it's not as easy a dependency to fulfill
        cp = subprocess.run(["/usr/bin/rsync",
                             "-r", "--compress", "--partial", "--progress", "--rsh", "ssh", "--delete", "--delete-after", "--force", "-g", "-l", "-o", "-p", "-v", "-S", "-t", "--timeout=8",
                             pomXmlToProcess.javaDocSyncTarget.syncSource,
                             "%s@%s" % (pomXmlToProcess.javaDocSyncTarget.repoUser, pomXmlToProcess.javaDocSyncTarget.syncTarget)]) # , "-n"])
        if cp.returncode != 0:
            log.error("Failed to rsync -r --compress --partial --progress --rsh ssh --delete --delete-after --force -g -l -o -p -v -S -t --timeout=8 %s %s@%s ..." % (
                pomXmlToProcess.javaDocSyncTarget.syncSource,
                pomXmlToProcess.javaDocSyncTarget.repoUser,
                pomXmlToProcess.javaDocSyncTarget.syncTarget))
            sys.exit(1) # or continue?
        log.good("Successfully performed rsync %s %s@%s ..." % (
                pomXmlToProcess.javaDocSyncTarget.syncSource,
                pomXmlToProcess.javaDocSyncTarget.repoUser,
                pomXmlToProcess.javaDocSyncTarget.syncTarget))

    # chown?
    
    log.good("Successfully published javadoc for %s" % pomXmlPath);
    
    sys.exit(0)
